/**************************************************************************//**
 * @file     pga.c
 * @version  V1.00
 * $Revision: 5 $
 * $Date: 16/03/29 4:34p $
 * @brief    NM1120 PGA driver source file
 *
 * @note
 * Copyright (C) 2016 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/
#include "NM1120.h"

/** @addtogroup NM1120_Device_Driver NM1120 Device Driver
  @{
*/

/** @addtogroup NM1120_PGA_Driver PGA Driver
  @{
*/


/** @addtogroup NM1120_PGA_EXPORTED_FUNCTIONS PGA Exported Functions
  @{
*/

/**
  * @brief      Enable PGA module with gain value and output channel
  * @param[in]  pga     The pointer of the specified PGA module
  * @param[in]  u32Gain is gain index. Including :
  *             - \ref PGA_GAIN_1
  *             - \ref PGA_GAIN_2
  *             - \ref PGA_GAIN_4
  *             - \ref PGA_GAIN_6
  *             - \ref PGA_GAIN_8
  *             - \ref PGA_GAIN_10
  *             - \ref PGA_GAIN_12
  *             - \ref PGA_GAIN_16
  * @param[in]  u32OutputMask is PGA output channels mask. Including :
  *             - \ref PGA_CTL_PGAADEN_Msk
  *             - \ref PGA_CTL_PGAOUTEN_Msk
  *             - \ref PGA_CTL_PGACMPEN_Msk
  * @return  None
  */
void PGA_Open(PGA_T *pga, uint32_t u32Gain, uint32_t u32OutputMask)
{
    pga->CTL = PGA_CTL_PGAEN_Msk
             | (u32Gain << PGA_CTL_GAIN_Pos)
             | (u32OutputMask);
}

/**
  * @brief      Disable PGA module
  * @param[in]  pga     The pointer of the specified PGA module
  * @return     None
  */
void PGA_Close(PGA_T *pga)
{
    pga->CTL = pga->CTL & (~PGA_CTL_PGAEN_Msk);
}

/*@}*/ /* end of group NM1120_PGA_EXPORTED_FUNCTIONS */

/*@}*/ /* end of group NM1120_PGA_Driver */

/*@}*/ /* end of group NM1120_Device_Driver */

/*** (C) COPYRIGHT 2016 Nuvoton Technology Corp. ***/
