/******************************************************************************
 * @file     main.c
 * @version  V1.00
 * $Revision: 3 $
 * $Date: 16/03/31 7:18p $
 * @brief    Show how to use USCI_I2C interface to access EEPROM.
 * @note
 * Copyright (C) 2016 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/
#include <stdio.h>
#include "NM1120.h"


/*---------------------------------------------------------------------------------------------------------*/
/* Global variables                                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
uint8_t g_u8DeviceAddr;
uint8_t g_au8TxData[3];
volatile uint8_t g_u8RxData;
volatile uint8_t g_u8DataLenM;
volatile uint8_t g_u8EndFlagM = 0;
enum USCI_I2C_MASTER_EVENT m_Event;

typedef void (*USCI_I2C_FUNC)(uint32_t u32Status);

static USCI_I2C_FUNC s_USCI_I2CMHandlerFn = NULL;

/*---------------------------------------------------------------------------------------------------------*/
/*  USCI_I2C1 IRQ Handler                                                                                       */
/*---------------------------------------------------------------------------------------------------------*/
void USCI1_IRQHandler(void)
{
    uint32_t u32Status;

    u32Status = USCI_I2C_GET_PROT_STATUS(USCI1);

    if (USCI_I2C_GET_TIMEOUT_FLAG(USCI1)) {
        /* Clear USCI_I2C1 Timeout Flag */
        USCI_I2C_ClearTimeoutFlag(USCI1);
    } else {
        if (s_USCI_I2CMHandlerFn != NULL)
            s_USCI_I2CMHandlerFn(u32Status);
    }
}

/*---------------------------------------------------------------------------------------------------------*/
/*  USCI_I2C Tx Callback Function                                                                               */
/*---------------------------------------------------------------------------------------------------------*/
void USCI_I2C_EEPROM_MasterTx(uint32_t u32Status)
{
    if((u32Status & USCI_PROTSTS_STARIF_Msk) == USCI_PROTSTS_STARIF_Msk) {
        USCI_I2C_CLR_PROT_INT_FLAG(USCI1, USCI_PROTSTS_STARIF_Msk); /* Clear START INT Flag */
        USCI_I2C_SET_DATA(USCI1, (g_u8DeviceAddr << 1) | 0x00); /* Write SLA+W to Register TXDAT */
        m_Event = MASTER_SEND_ADDRESS;
        USCI_I2C_SET_CONTROL_REG(USCI1, USCI_I2C_CTL_PTRG);
    } else if((u32Status & USCI_PROTSTS_ACKIF_Msk) == USCI_PROTSTS_ACKIF_Msk) {
        USCI_I2C_CLR_PROT_INT_FLAG(USCI1, USCI_PROTSTS_ACKIF_Msk);  /* Clear ACK INT Flag */
        if(m_Event == MASTER_SEND_ADDRESS) {
            USCI_I2C_SET_DATA(USCI1, g_au8TxData[g_u8DataLenM++]);  /* SLA+W has been transmitted and write ADDRESS to Register TXDAT */
            m_Event = MASTER_SEND_DATA;
            USCI_I2C_SET_CONTROL_REG(USCI1, USCI_I2C_CTL_PTRG);
        } else if(m_Event == MASTER_SEND_DATA) {
            if(g_u8DataLenM != 3) {
                USCI_I2C_SET_DATA(USCI1, g_au8TxData[g_u8DataLenM++]);  /* ADDRESS has been transmitted and write DATA to Register TXDAT */
                USCI_I2C_SET_CONTROL_REG(USCI1, USCI_I2C_CTL_PTRG);
            } else {
                g_u8EndFlagM = 1;
                m_Event = MASTER_STOP;
                USCI_I2C_SET_CONTROL_REG(USCI1, (USCI_I2C_CTL_PTRG | USCI_I2C_CTL_STO));        /* Send STOP signal */
            }
        }
    } else if((u32Status & USCI_PROTSTS_NACKIF_Msk) == USCI_PROTSTS_NACKIF_Msk) {
        USCI_I2C_CLR_PROT_INT_FLAG(USCI1, USCI_PROTSTS_NACKIF_Msk); /* Clear NACK INT Flag */
        g_u8EndFlagM = 0;
        if(m_Event == MASTER_SEND_ADDRESS) {    /* SLA+W has been transmitted and NACK has been received */
            m_Event = MASTER_SEND_START;
            USCI_I2C_SET_CONTROL_REG(USCI1, (USCI_I2C_CTL_PTRG | USCI_I2C_CTL_STA));        /* Send START signal */
        } else if(m_Event == MASTER_SEND_DATA) {    /* ADDRESS has been transmitted and NACK has been received */
            m_Event = MASTER_STOP;
            USCI_I2C_SET_CONTROL_REG(USCI1, (USCI_I2C_CTL_PTRG | USCI_I2C_CTL_STO));        /* Send STOP signal */
        } else {
            printf("Get Wrong NACK Event\n");
        }
    }
}

/*---------------------------------------------------------------------------------------------------------*/
/*  USCI_I2C Rx Callback Function                                                                               */
/*---------------------------------------------------------------------------------------------------------*/
void USCI_I2C_EEPROM_MasterRx(uint32_t u32Status)
{
    if((u32Status & USCI_PROTSTS_STARIF_Msk) == USCI_PROTSTS_STARIF_Msk) {
        USCI_I2C_CLR_PROT_INT_FLAG(USCI1, USCI_PROTSTS_STARIF_Msk); /* Clear START INT Flag */
        if(m_Event == MASTER_SEND_START) {
            USCI_I2C_SET_DATA(USCI1, (g_u8DeviceAddr << 1) | 0x00); /* Write SLA+W to Register TXDAT */
            m_Event = MASTER_SEND_ADDRESS;
        } else if(m_Event == MASTER_SEND_REPEAT_START) {
            USCI_I2C_SET_DATA(USCI1, (g_u8DeviceAddr << 1) | 0x01); /* Write SLA+R to Register TXDAT */
            m_Event = MASTER_SEND_H_RD_ADDRESS;
        }
        USCI_I2C_SET_CONTROL_REG(USCI1, USCI_I2C_CTL_PTRG);
    } else if((u32Status & USCI_PROTSTS_ACKIF_Msk) == USCI_PROTSTS_ACKIF_Msk) {
        USCI_I2C_CLR_PROT_INT_FLAG(USCI1, USCI_PROTSTS_ACKIF_Msk);  /* Clear ACK INT Flag */
        if(m_Event == MASTER_SEND_ADDRESS) {
            USCI_I2C_SET_DATA(USCI1, g_au8TxData[g_u8DataLenM++]);  /* SLA+W has been transmitted and write ADDRESS to Register TXDAT */
            m_Event = MASTER_SEND_DATA;
            USCI_I2C_SET_CONTROL_REG(USCI1, USCI_I2C_CTL_PTRG);
        } else if(m_Event == MASTER_SEND_DATA) {
            if(g_u8DataLenM != 2) {
                USCI_I2C_SET_DATA(USCI1, g_au8TxData[g_u8DataLenM++]);  /* ADDRESS has been transmitted and write DATA to Register TXDAT */
                USCI_I2C_SET_CONTROL_REG(USCI1, USCI_I2C_CTL_PTRG);
            } else {
                m_Event = MASTER_SEND_REPEAT_START;
                USCI_I2C_SET_CONTROL_REG(USCI1, (USCI_I2C_CTL_PTRG | USCI_I2C_CTL_STA));    /* Send repeat START signal */
            }
        } else if(m_Event == MASTER_SEND_H_RD_ADDRESS) {
            m_Event = MASTER_READ_DATA;
            USCI_I2C_SET_CONTROL_REG(USCI1, USCI_I2C_CTL_PTRG);
        }
    } else if((u32Status & USCI_PROTSTS_NACKIF_Msk) == USCI_PROTSTS_NACKIF_Msk) {
        USCI_I2C_CLR_PROT_INT_FLAG(USCI1, USCI_PROTSTS_NACKIF_Msk); /* Clear NACK INT Flag */
        if(m_Event == MASTER_SEND_ADDRESS) {
            m_Event = MASTER_SEND_START;
            USCI_I2C_SET_CONTROL_REG(USCI1, (USCI_I2C_CTL_PTRG | USCI_I2C_CTL_STA));    /* Send START signal */
        } else if(m_Event == MASTER_READ_DATA) {
            g_u8RxData = (unsigned char) USCI_I2C_GET_DATA(USCI1) & 0xFF;
            g_u8EndFlagM = 1;
            m_Event = MASTER_STOP;
            USCI_I2C_SET_CONTROL_REG(USCI1, (USCI_I2C_CTL_PTRG | USCI_I2C_CTL_STO));    /* DATA has been received and send STOP signal */
        } else {
            printf("Get Wrong NACK Event\n");
        }
    }
}

void SYS_Init(void)
{
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable 48MHz HIRC */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRCEN_Msk);

    /* Waiting for 48MHz clock ready */
    CLK_WaitClockReady(CLK_STATUS_HIRCSTB_Msk);

    /* HCLK Clock source from HIRC */
    CLK_SetHCLK(CLK_HCLK_SRC_HIRC,CLK_CLKDIV_HCLK(1));

    /* Enable IP clock */
    CLK_EnableModuleClock(USCI0_MODULE);
    CLK_EnableModuleClock(USCI1_MODULE);


    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate SystemCoreClock and cyclesPerUs automatically. */
    SystemCoreClockUpdate();

    /* USCI-Uart0-GPD5(TX) + GPD6(RX) */
    /* Set GPD multi-function pins for USCI UART0 GPD5(TX) and GPD6(RX) */
    SYS->GPD_MFP = SYS->GPD_MFP & ~(SYS_GPD_MFP_PD5MFP_Msk | SYS_GPD_MFP_PD6MFP_Msk) | (SYS_GPD_MFP_PD5_UART0_TXD | SYS_GPD_MFP_PD6_UART0_RXD);

    /* Set GPD5 as output mode and GPD6 as Input mode */
    GPIO_SetMode(PD, BIT5, GPIO_MODE_OUTPUT);
    GPIO_SetMode(PD, BIT6, GPIO_MODE_INPUT);

    /* Set GPC multi-function pins for USCI I2C1 GPC0(SCL) and GPC2(SDA) */
    SYS->GPC_MFP = SYS->GPC_MFP & ~(SYS_GPC_MFP_PC0MFP_Msk | SYS_GPC_MFP_PC2MFP_Msk) | (SYS_GPC_MFP_PC0_I2C1_SCL | SYS_GPC_MFP_PC2_I2C1_SDA);

    /* Lock protected registers */
    SYS_LockReg();
}

void USCI_I2C1_Init(void)
{
    /* Open USCI_I2C1 and set clock to 100k */
    USCI_I2C_Open(USCI1, 100000);

    /* Get USCI_I2C1 Bus Clock */
    printf("USCI I2C clock %d Hz\n", USCI_I2C_GetBusClockFreq(USCI1));

    /* Set USCI_I2C1 Slave Addresses */
    USCI_I2C_SetSlaveAddr(USCI1, 0, 0x15, USCI_I2C_GCMODE_DISABLE);   /* Slave Address : 0x15 */

    USCI_I2C_ENABLE_PROT_INT(USCI1, (USCI_PROTIEN_ACKIEN_Msk | USCI_PROTIEN_NACKIEN_Msk | USCI_PROTIEN_STORIEN_Msk | USCI_PROTIEN_STARIEN_Msk));
    NVIC_EnableIRQ(USCI1_IRQn);
}

int main()
{
    uint32_t i;

    SYS_Init();

    /* Init USCI UART0 to 115200-8n1 for print message */
    USCI_UART_Open(USCI0, 115200);

    /*
        This sample code sets I2C bus clock to 100kHz. Then, accesses EEPROM 24LC64 with Byte Write
        and Byte Read operations, and check if the read data is equal to the programmed data.
    */

    printf("+-------------------------------------------------------+\n");
    printf("|     USCI_I2C Driver Sample Code with EEPROM 24LC64    |\n");
    printf("+-------------------------------------------------------+\n");

    /* Init USCI_I2C1 to access EEPROM */
    USCI_I2C1_Init();

    g_u8DeviceAddr = 0x50;

    for (i = 0; i < 0x100; i++) {
        g_au8TxData[0] = (uint8_t)((i & 0xFF00) >> 8);
        g_au8TxData[1] = (uint8_t)(i & 0x00FF);
        g_au8TxData[2] = (uint8_t)(g_au8TxData[1] + 3);

        g_u8DataLenM = 0;
        g_u8EndFlagM = 0;

        /* USCI_I2C function to write data to slave */
        s_USCI_I2CMHandlerFn = (USCI_I2C_FUNC)USCI_I2C_EEPROM_MasterTx;

        /* USCI_I2C as master sends START signal */
        m_Event = MASTER_SEND_START;
        USCI_I2C_SET_CONTROL_REG(USCI1, USCI_I2C_CTL_STA);

        /* Wait USCI_I2C Tx Finish */
        while (g_u8EndFlagM == 0);
        g_u8EndFlagM = 0;

        /* USCI_I2C function to read data from slave */
        s_USCI_I2CMHandlerFn = (USCI_I2C_FUNC)USCI_I2C_EEPROM_MasterRx;

        g_u8DataLenM = 0;
        g_u8DeviceAddr = 0x50;

        m_Event = MASTER_SEND_START;
        USCI_I2C_SET_CONTROL_REG(USCI1, USCI_I2C_CTL_STA);

        /* Wait USCI_I2C Rx Finish */
        while (g_u8EndFlagM == 0);
        g_u8EndFlagM = 0;

        /* Compare data */
        if (g_u8RxData != g_au8TxData[2]) {
            printf("USCI_I2C Byte Write/Read Failed, Data 0x%x\n", g_u8RxData);
            return -1;
        }
    }
    printf("USCI_I2C Access EEPROM Test OK\n");

    while(1);
}

/*** (C) COPYRIGHT 2016 Nuvoton Technology Corp. ***/
